@@ -13,7 +13,7 @@ public:
 	DnsRecord(struct in_addr ip, string& hostName, int serial, time_t last_access);
 	DnsRecord(struct in_addr ip, string hostName);
     friend ostream& operator<<(ostream &os, const DnsRecord *cr) {
-		os << "src " << inet_ntoa(cr->ip_m)
+		os << "src " << inet_to_string(cr->ip_m)
 			<< " host " << cr->hostName_m
 			<< " serial " << cr->serial_m
 			<< " last " << ctime(&cr->last_access_m);
@@ -41,8 +41,11 @@ class DnsCache {
 	DnsRecordMap	ipToRecord_m;
 	DnsRecordsByDate	recordsByDate_m;
 	int	expiryTime_m;
+	pthread_mutex_t mutex_;
 public:
-	DnsCache(int expiryTime) : expiryTime_m(expiryTime) { }
+	DnsCache(int expiryTime) : expiryTime_m(expiryTime) {
+		pthread_mutex_init(&mutex_, NULL);
+	}
 	// find the host from the IP 
 	int	FindHostFromIP(const struct in_addr& ip, const char *mac);
 	void FlushOld();
