@@ -28,27 +28,48 @@ using namespace std;
 #include "sll.h"
 #include "dns.h"
 #include "prio.h"
+#include "monlist.h"
 
-static bool	FlushDataToDb(RTable *rt, int since)
-{
-	struct timeval tv;
-	bool seen = false;
-	CRecordList	crl;
+// TODO: put this in the RTable struucture
 
-	gettimeofday(&tv, NULL);
-	tv.tv_sec -= since;
-	rt->MakeListOfFlushItems(&crl, &tv);
-	for (CRecordListIter iter = crl.begin(); iter != crl.end(); ) {
-		CRecordListIter prev = iter;
-		CRecord *cr = *iter++;
-		cr->srcSerial_m = rt->FindHostFromIP(cr->ip_src_m, cr->llSrc_m);
-		cr->dstSerial_m = rt->FindHostFromIP(cr->ip_dst_m, cr->llDst_m);
-		if (SQlSaveRecord(cr, rt->Get_InterfaceName()) != 0) {
-			seen = true;
-			crl.erase(prev);
-			delete cr;
-		} 
+static struct bpf_program compiledSecondaryFilter;
+static bool	secondaryFilterCompiled = false;
+
+int	FlushDataToDb(RTable  *acom, int sinceInSeconds) {
+
+	int seen = 0;
+
+	PrioRecordList	*crl = new PrioRecordList();
+	
+	if (sinceInSeconds == 0) {
+		acom->MakeListOfFlushItems(crl);
+	} else {
+		struct timeval tv;
+		gettimeofday(&tv, NULL);
+		tv.tv_sec -= sinceInSeconds;
+		acom->MakeListOfFlushItems(crl, &tv);
 	}
+	for (PrioRecordListIter iter = crl->begin(); iter != crl->end(); ) {
+		PrioRecordListIter prev = iter;
+		CRecord *cr = static_cast<CRecord *>(*iter++);
+
+		cr->srcSerial_m = acom->FindHostFromIP(cr->ip_src_m, cr->llSrc_m);
+		cr->dstSerial_m = acom->FindHostFromIP(cr->ip_dst_m, cr->llDst_m);
+		//cr->Print(cout);
+		if (sinceInSeconds == 0)
+			cr->flushed_m = 1;
+		else
+			cr->flushed_m = 0;
+		if (SQlSaveRecord(cr, acom->Get_InterfaceName()) != 0) {
+			//cout << "OK" << endl;
+			seen++;
+			crl->erase(prev);
+			delete cr;
+		}  else {
+			cout << "NOT OK" << endl;
+		}
+	}
+	delete crl;
 	return seen;
 }
 
@@ -80,9 +101,30 @@ static void    *SaveDataThread(void *arg)
 	}
 }
 
+static void    *FullFlushyThread(void *arg)
+{
+	RTable	*rt = (RTable *)arg;
+
+	for (;;) {
+		time_t	ti;
+		time(&ti);
+
+		if (ti >= rt->nextFlush_) {
+			rt->nextFlush_ += rt->fullFlushInterval_;
+			//cout << "Periodic flush" << ctime(&ti);
+			int ii = FlushDataToDb(rt, 0);
+			syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "%d items flushed by periodic flusher", ii);
+			rt->Get_DnsCache()->FlushOld();
+		} 
+		sleep(1);
+		pthread_testcancel();
+
+	}
+}
+
 
 u_char *handle_IP(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char * packet, int hrdSize,
-	const char *llSrc, const char *llDst)
+	const char *llSrc, const char *llDst, bool save)
 {
   const struct my_ip *ip;
   u_int length = pkthdr->len;
@@ -127,8 +169,8 @@ u_char *handle_IP(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *
 
 	RTable  *rt = (RTable *)args;
 
-	CRecord	*cr = new CRecord(ip, pkthdr->len, llSrc, llDst);
-	rt->Add(cr, pkthdr->ts);
+	CRecord	cr(ip, pkthdr->len, llSrc, llDst);
+	rt->Accumulate(&cr, pkthdr->ts); //, save ? (int)length : -1, (unsigned char *)packet);
 	return NULL;
 }
 
@@ -141,7 +183,22 @@ void my_callback(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *p
 	RTable *rt = (RTable *)args;
 	u_int caplen = pkthdr->caplen;
 	u_int length = pkthdr->len;
+	bool save = false;
 
+//	FlushDataToDb(rt, rt->Get_Age());
+//	rt->Get_DnsCache()->FlushOld();
+
+	if (secondaryFilterCompiled) {
+#if 0
+		if (bpf_filter(compiledSecondaryFilter.bf_insns,
+				(u_char *)packet,
+				pkthdr->len,
+				pkthdr->caplen) != 0) {
+			cout << "packet recognised by secondary filter" << endl;
+#endif
+			save = true;
+//		}
+	}
 	switch (rt->Get_DataLinkType()) {
 	case DLT_PPP:
 		break;
@@ -151,7 +208,7 @@ void my_callback(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *p
 			u_short ether_type = ntohs(sllp->sll_protocol);
 			switch (ether_type) {
 			case ETHERTYPE_IP: {
-					handle_IP(args, pkthdr, packet, SLL_HDR_LEN, NULL, NULL);
+					handle_IP(args, pkthdr, packet, SLL_HDR_LEN, NULL, NULL, save);
 					break;
 				}
 			default:
@@ -178,7 +235,7 @@ void my_callback(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *p
 			case ETHERTYPE_IP:
 				ether_ntoa_r((struct ether_addr *)eptr->ether_shost, llSrc);
 				ether_ntoa_r((struct ether_addr *)eptr->ether_dhost, llDst);
-				handle_IP(args, pkthdr, packet, sizeof (struct ether_header), llSrc, llDst); //llSrc, llDst);
+				handle_IP(args, pkthdr, packet, sizeof (struct ether_header), llSrc, llDst, save);
 				break;
 			case ETHERTYPE_ARP:
 				break;
@@ -197,11 +254,14 @@ void my_callback(u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *p
 
 RTable   *rt;
 pthread_t	writeThread;
+pthread_t	flushyThread;
  
 
 static void	FinalFlush()
 {
-	syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "got signal waiting for child to die off");
+//	syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "got signal waiting for child to die off");
+	if (flushyThread)
+		pthread_cancel(flushyThread);
 	pthread_cancel(writeThread);
 	
 	syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "flushing");
@@ -224,7 +284,7 @@ static void termination_handler(int signum)
 		}
 	}
 	FinalFlush();
-	syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "%d recived and %d dropped", ps.ps_recv, ps.ps_drop);
+	syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "%d received and %d dropped", ps.ps_recv, ps.ps_drop);
 	exit(0);
 }
 
@@ -238,15 +298,20 @@ int main(int argc, char *argv[])
 	int	opt;
 	int	sltime = 10;
 	int	numPackets = 10;
-	int	age = 30;
+	int	age = 120;
 	char	*connectionString = "usage@localhost";
-	char	*filter = NULL;
-	int		dnsExpiryTime = 30;
+	char	*filter = 0;
+	char	*secondaryFilter = 0;
+	int		dnsExpiryTime = 120;
 	int	backGround = false;
 	int	ifDownSleep = 10;
+	int	fullFlushOnMinDiv = 0;
 
-	while ((opt = getopt(argc, argv, "S:bx:s:p:c:f:d:i:a:")) != -1) {
+	while ((opt = getopt(argc, argv, "F:S:bx:s:p:c:f:e:d:i:a:")) != -1) {
 		switch (opt) {
+		case 'F':
+			fullFlushOnMinDiv = atoi(optarg);
+			break;
 		case 'S':
 			ifDownSleep = atoi(optarg);
 			break;
@@ -268,6 +333,9 @@ int main(int argc, char *argv[])
 		case 'f':
 			filter = optarg;
 			break;
+		case 'e':
+			secondaryFilter = optarg;
+			break;
 		case 'd':
 		case 'i':
 			dev = optarg;
@@ -277,11 +345,14 @@ int main(int argc, char *argv[])
 			break;
 		default:
 			cerr << "invalid usage" << endl;
+			cerr << "-F seconds - fully flush the usage list with this seconds divisor" << endl;
+			cerr << " for example, 10 - every 10 minutes but at 1:10, 1:20 etc" << endl;
 			cerr << "-b - fork in background and detatch from console" << endl;
 			cerr << "-x dns expiry time: the time an entry lives int he dns cache" << endl;
 			cerr << "-s sleep_time: the time the write sleeps for" << endl;
 			cerr << "-p num_packets: the number of packets to collect, 0 for forever" << endl;
 			cerr << "-f filter: pcap filter argument" << endl;
+			cerr << "-e seconday filter: filter records to save the whole data" << endl;
 			cerr << "-d or -i: device (or interface) to capure on" << endl;
 			cerr << "-a: age of to/from packets inactivity before writing to db" << endl;
 			cerr << "-c: connection string" << endl;
@@ -318,26 +389,30 @@ int main(int argc, char *argv[])
 		cerr << "pcap_lookupnet(): " << errbuf << endl;
 		exit(1);
 	}
-	struct bpf_program compiledFilter;
-
-	if (filter != 0) {
-		if (pcap_compile(descr, &compiledFilter, filter, 0, netp) == -1) {
-			cerr << "Error calling pcap_compile" << endl;
-			exit(1);
-		}
-	}
 
 	DnsCache	dnsCache(dnsExpiryTime);
-	rt = new RTable(sltime, &dnsCache, age, dev);
+	rt = new RTable(sltime, &dnsCache, age, fullFlushOnMinDiv, dev);
 
+#if 1
    if (pthread_create(&writeThread, NULL, SaveDataThread, (void *)rt) < 0) {
 		perror("pthread_create");
 		exit(1);
     }
+#endif
+	if (fullFlushOnMinDiv != 0) {
+		if (pthread_create(&flushyThread, NULL, FullFlushyThread, (void *)rt) < 0) {
+			perror("pthread_create for flushy");
+			exit(1);
+		}
+	}
 	signal(SIGINT, termination_handler);
 	signal(SIGTERM, termination_handler);
 
+	bool	linkFilterCompiled = false;
+
 	for (;;) {
+		struct bpf_program compiledFilter;
+		
 		if ((descr = pcap_open_live(dev, BUFSIZ, 1, -1, errbuf)) == (pcap_t *)0) {
 			syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "pcap_open_live(): %s", errbuf);
 			if (numPackets != 0)
@@ -345,6 +420,29 @@ int main(int argc, char *argv[])
 		} else {
 			syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "capturing datalink type %d", pcap_datalink(descr));
 			rt->Set_tcapInfo(descr);
+
+			if (filter != 0 && !linkFilterCompiled) {
+				if (pcap_compile(descr, &compiledFilter, filter, 0, netp) == -1) {
+					cerr << "Error calling pcap_compile" << endl;
+					syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "Error calling pcap_compile");
+					exit(1);
+				}
+				linkFilterCompiled = true;
+			}
+			if (secondaryFilter != 0 && !secondaryFilterCompiled) {
+				if (pcap_compile(descr, &compiledSecondaryFilter, secondaryFilter, 0, netp) == -1) {
+					cerr << "Error calling pcap_compile for secondary filter" << endl;
+					syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "Error calling pcap_compile for secondary");
+					exit(1);
+				} else {
+					cerr << "secondary filter '" << secondaryFilter << "' compiled OK" << endl;
+					syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_INFO), "secondary filter compiled OK");
+					
+				}
+				secondaryFilterCompiled = true;
+				
+			}
+
 			if (filter != 0) {
 				/* set the compiled program as the filter */
 				if (pcap_setfilter(descr, &compiledFilter) == -1) {
